import { extname } from 'path';
import acorn from 'acorn-jsx';
import { walk } from 'estree-walker';
import { createFilter } from 'rollup-pluginutils';
import MagicString from 'magic-string';
import escodegen from 'escodegen';

function convert(ast) {
  var trueExpression = undefined;
  var falseExpression = undefined;
  var condition = escodegen.generate(ast.test);

  if (ast.consequent.type === 'ConditionalExpression') {
    trueExpression = convert(ast.consequent);
  } else {
    trueExpression = escodegen.generate(ast.consequent);
  }

  if (ast.alternate.type === 'ConditionalExpression') {
    falseExpression = convert(ast.alternate);
  } else {
    falseExpression = escodegen.generate(ast.alternate);
  }

  return 'if (' + condition + ') {\n' + trueExpression + '\n} else {\n' + falseExpression + '\n}';
}

function plugin() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var extensions = options.extensions || ['.js'];
  var filter = createFilter(options.include, options.exclude);
  var sourceMap = options.sourceMap !== false;
  var extra = options.extra !== false;

  return {
    transform: function transform(code, id) {
      if (!filter(id)) {
        return null;
      }
      if (extensions.indexOf(extname(id)) === -1) {
        return null;
      }

      var ast = undefined;

      try {
        ast = acorn.parse(code, {
          ecmaVersion: 6,
          sourceType: 'module',
          plugins: {
            jsx: true
          }
        });
      } catch (err) {
        err.message += ' in ' + id;
        throw err;
      }

      var magicString = new MagicString(code);

      walk(ast, {
        enter: function enter(node) {
          if (node.type === 'ConditionalExpression') {
            if (extra && node.consequent.type !== 'ConditionalExpression') {
              return;
            }

            if (sourceMap) {
              magicString.addSourcemapLocation(node.start);
              magicString.addSourcemapLocation(node.end);
            }

            // console.log(`\nfile: ${id}`);
            var converted = convert(node);
            magicString.overwrite(node.start, node.end, converted);
            return;
          }
        }
      });

      var resultCode = magicString.toString();
      var map = sourceMap ? magicString.generateMap() : null;

      return {
        code: resultCode,
        map: map
      };
    }
  };
}

export default plugin;